<?php

/**
* This file includes the main class fpor the dynamic
* menu plugin.
*
* PLUGIN: Dynamic Menu
*
* @package DynPG PlugIn Engine
* @author Daniel Schliebner*
*/

if ( !defined('CASE_LOWER') ) {
    define ( 'CASE_LOWER' , 0 );
}

if ( !defined('CASE_UPPER') ) {
    define ( 'CASE_UPPER' , 1 );
}

class dynPG_Dynamic_Menu {

    // $property_code -> string
    // String which is generatet by DynPG where Plugin might by inserted in.
    var $property_code;

    // $dynpg_instance -> instance of class TDynPG_MainClass
    // Copy of instance from super global $GLOBALS['DynPG'].
    var $dynpg_instance;

    // $rootToTool -> string
    // Path to the CMS directory in which DynPG has been installed
    var $rootToTool;

    // $rootToPlugin -> string
    // Path to the plugin directory in which plugin
    var $rootToPlugin;

    // $output_Message -> string
    // buffered outputs when succeeded or failed
    var $output_Message;

    // $language -> array[0..1] of [ string | TLanguage ]
    // instaance for current language class (default is "de")
    var $language;

    // $template -> TTemplate
    // instance for current template processor
    var $template;

    // $parameters -> array
    // array of parameters for this plugin
    var $parameters;

    function dynPG_Dynamic_Menu() {
        // constructor //

        // 1. initiate variables with default values

        $this->output_Message = '';
        $this->property_code  = '';


        // 2. get some foreign data of DynPG CMS

        if ( isset($GLOBALS['DynPG']) && is_object($GLOBALS['DynPG']) ) {
            $this->dynpg_instance = $GLOBALS['DynPG'];
            $this->rootToTool     = $this->dynpg_instance->PathToRoot;
            $this->rootFromPlugin = dynPG_Dynamic_Menu::realpath($this->rootToTool . 'plugins/DPGdynmenu/') . '/';
        } else {
            $this->rootToTool     = '';
            $this->rootToPlugin   = '';
            $this->dynpg_instance = null;
        }


        // 3. read current language (selected by includes class)

        $this->setupLanguage($this->dynpg_instance->_PlugIn['dpgdynmenu']['language']);


        // 4. instanciate current template processor

        // a) create instance

        $this->template = new TTemplate();

        // b) set some default values

        $this->template->addVar('Main->rootToTool', $this->rootToTool );
        $this->template->addVar('Main->QueryString', dynpg_htmlentities_rigid(((!getenv("QUERY_STRING")) ? $_SERVER['QUERY_STRING'] : getenv("QUERY_STRING")),ENT_COMPAT,true) );
        $this->template->addVar('Main->Host', dynpg_htmlentities_rigid(((!getenv("HTTP_HOST")) ? $_SERVER['HTTP_HOST'] : getenv("HTTP_HOST")),ENT_COMPAT,true) );
        $this->template->addVar('Main->pathToPlugin', 'plugins/DPGdynmenu/' );
        $this->template->addVar('Main->time', time() );
        $this->template->addVar('Main->Self', $_SERVER['PHP_SELF'] );

        register_shutdown_function(array(&$this, 'destroy_dynPG_Dynamic_Menu'));


        // 5. connect MySQL

        $this->establish_MySQL_Connection();


        // 6. Include required data

        $this->setupRequirements();


        // 7. read parameters

        // a) instanciate defaults

            $_ = Array(

                        'order' => Array('INDEX'),

                        'sort' => Array('ASC'),

						'order_articles' => Array('INDEX'),

                        'sort_articles' => Array('ASC'),

                        'href_page_groups' => null,

                        'href_page_articles' => null,

                        'href_page_detail' => null,

                        'href_subpages' => Array(),

                        'full_load' => false,

                        'full_load_articles' => false,

                        'load_articles' => true,

                        'limit_articles' => false,

                        'expand_subtree' => Array(),

                        'implode_subtree' => Array(),

                        'real_expand' => false,

                        // @parameter "speed_mode"
                        // The speed_mode parameter can increase the menu speed
                        // in enourmous ways. You only have to select the correct
                        // mode. Espacially in multi-menu pages, this parameter
                        // is very useful.
						// - false = off,
						// - 1 	 = only one depth more than current,
						// - 2 	 = NO items before currently selected item,
						// - 3 	 = 1 & 2,
						// - 4 	 = 2 & only current depth.
						'speed_mode' => false,

						// @parameter "parent_select"
                        // If true, "parent_select" indicates, that the currently
                        // selected group's parent is taken as root for the
						// tree-building process, otherwise the group itself.
						'parent_select' => true,

                        // @parameter "max_depth"
                        // Upper bound for a group depth.
						// - null = infinity,
                        // - int  = 0 <= max_depth < infinity.
                        // NOTE: max_depth overwrites speed_mode settings, that
                        // influence the max_depth value.
						'max_depth' => null,

						// @parameter "min_depth"
						// Lower bound for a group depth.
                        // - null = infinity,
                        // - int  = 0 <= min_depth < infinity.
                        // NOTE: min_depth overwrites speed_mode settings, that
                        // influence the min_depth value.
						'min_depth' => null,

						// @parameter "fetch_only"
						// Fetches only one group, including subgroups.
                        // - null = infinity,
                        // - int  = 0 <= fetch_only < infinity.
						'fetch_only' => null,

                        // @parameter "parse_article_lead"
                        // If true, "parse_article_lead" indicates, that text replacing
                        // will be performed on not empty field "lead" of article.
                        // This can slow down the menu, because of recursive call of
                        // PerformTextReplacement() on all found articles.
                        'parse_article_lead' => false,

                        // @parameter "parse_group_shorttext"
                        // If true, "parse_group_shorttext" indicates, that text replacing
                        // will be performed on not empty field "shorttext" of group.
                        // This can slow down the menu, because of recursive call of
                        // PerformTextReplacement() on all found groups.
                        'parse_group_shorttext' => false,

                        // @parameter "hide_not_public_groups"
                        // If true, "hide_not_public_groups" indicates, that not published groups
                        // and all his subgroups are not returned bye writeMenu
                        'hide_not_public_groups' => true,

                        // @parameter "hide_locked_groups_by_community"
                        // If true, "hide_locked_groups_by_community" indicates,
                        // that groupes, locked by the community modul are hidden
                        // unless the user id logged in
                        'hide_locked_groups_by_community' => false

                      );

        // b) read user defined

            $this->parameters = array_merge($_, $this->dynpg_instance->_PlugIn['dpgdynmenu']['parameters']);


		// 8. disable auto rendering

			$this->template->autoRenderOff();
    }

    function array_change_key_case($arr, $case = CASE_LOWER) {
        if (! is_array($arr) ) {
            return $arr;
        }

        $_ = $arr;
        foreach ( $arr as $key => $value ) {
            $_[ $case == CASE_UPPER ? strtoupper($key) : strtolower($key) ] = $value;
        }

        return $_;
    }

    function htmlrealpath($path) {
        if (!function_exists('realpath')) {
        	return $path;
        }
		$_ = @realpath($path);

        return (
			($_ != false)?
			str_replace(realpath($_SERVER['DOCUMENT_ROOT']), '', $_):
			$path
		);
    }

    function realpath($path) {
        if (!function_exists('realpath')) {
        	return $path;
        }
		$_ = @realpath($path);

        return (
			($_ != false)?
			$_:
			$path
		);
	}

    function perform() {
        if ( strpos($this->property_code, 'alt="DYNMENU"') !== false && preg_match('/<img([^<>]+)alt="DYNMENU"([^<>]+)\/>/', $this->property_code) ) {
            $dynamic_Menu_HTML4   = $this->template->render($this->writeMenu(), false);
            $this->property_code  = preg_replace('/<img([^<>]+)alt="DYNMENU"([^<>]+)\/>/', $dynamic_Menu_HTML4, $this->property_code);
        }
    }

    function setupLanguage($lang = 'de') {
        if(preg_match('/[^a-zA-Z]/', $lang) == 1){
            $lang = 'de';
        }
        require_once dirname( __FILE__ ) . '/languages/'.strtolower($lang).'.lang.php';

        $this->language[0] = strtolower($lang);
        $this->language[1] = new TLanguage($this->language[0], 'DPGdynmenu');
    }

    function establish_MySQL_Connection() {
        require_once $this->rootToTool . 'connectdb.php';
    }

    function setupRequirements() {
        require_once( dirname( __FILE__ ) . '/__menu_engine.class.php' );
    }

    function setParam($param = '', $value = '') {
        $this->parameters[strtolower($param)] = $value;
    }

    function writeMenu() {
        $this->dynpg_instance->Private_ReconnectMySQL();

        // deduce current file upload folder url
			if ( !empty($GLOBALS["DYNPG_VAR_OUTSIDE_IMGURL"]) ) {
	            $_general_image_url = $GLOBALS["DYNPG_VAR_OUTSIDE_IMGURL"];
	        } else {
	            $_general_image_url = $this->dynpg_instance->PathToRoot . FORMS_IMAGES_DESTINATIONURL_ROOT;
	        }

        // manipulate parameter "order" for adding it to SQL code
			$this->parameters['order'] = array_map(create_function('$val', 'return strtoupper($val);'), $this->parameters['order']);

        // init menu processor
			$menuProc = new Tmenu();

        // deduce current group id
        	$self = null;
	        $subE = false;
			if ($this->dynpg_instance->IdxSelector == 'NULL' || empty($this->dynpg_instance->IdxSelector)) {
	        	$self = null;
	            $subE = false;
	        } else {
				if (in_array(strtolower($this->dynpg_instance->PageSelector), Array('groups', 'articles', 'home'))) {
					$grp = mysql_fetch_assoc(
						mysql_query("SELECT * FROM `dynpg_cms_groups` WHERE `id` = '".mysql_real_escape_string($this->dynpg_instance->IdxSelector)."'")
					);
					if (!$grp['REFID_IS_START']) {
			            $self = $this->dynpg_instance->IdxSelector;
			            $subE = false;
			        } else {
			        	$self = $menuProc->getParent2Article($grp['REFID'], 'GROUP');
			            $subE = $grp['REFID'];
					}
		        } else {
		            $self = $menuProc->getParent2Article($this->dynpg_instance->IdxSelector, 'GROUP');
		            $subE = $this->dynpg_instance->IdxSelector;
		        }
	        }

        // init vars
        	$menu = Array();
            $groupKeys = Array();
			$maxDepth = null;
	        $currentDepth = $menuProc->getDepth($self) - 1;
	        $fetch_only_depth = $this->parameters['fetch_only']? $menuProc->getDepth($this->parameters['fetch_only']): 0;

	        if (!$this->parameters['parent_select'])
	        	$currentDepth++;

        // set max depth, depending on the parmeters max_depth and speed_mode
			if (is_integer($this->parameters['max_depth'])) {
	        	$maxDepth = $this->parameters['max_depth'];
	        } else {
	        	$maxDepth = in_array($this->parameters['speed_mode'], Array(1,3))? ($currentDepth + 1): null;
				$maxDepth = $this->parameters['speed_mode'] == 4? $currentDepth: $maxDepth;
	        }

        // fetch all successors (between minimal depth and max depth, specified thourgh parameter or speed_mode)
			$menuProc->list_Successors(
				$menu,
				$this->parameters['speed_mode'] > 1?
					($this->parameters['parent_select']? $menuProc->getDirectParent($self): $self):
					$this->parameters['fetch_only'],
				$minDepth = ($this->parameters['speed_mode'] > 1? $currentDepth: $fetch_only_depth),
				$this->parameters['order'],
				$this->parameters['sort'],
				null,
				null,
				$maxDepth,
                0,
                $this->parameters['hide_not_public_groups'],
                $groupKeys
			);

		// fetch all items in groups to denie too busy db server
		// (caution: may exceed memory limits)
			// $mem_old = memory_get_usage();
			$_ = mysql_query(
				"SELECT DISTINCT
						A.`id`,
						A.`GROUP`,
						A.`TITLE`,
						A.`LEAD`,
						A.`DATE_UP`,
						A.`DATE_TO`,
						A.`TIME_UP`,
						A.`TIME_TO`,
						A.`ISATHOME`,
						A.`CREATEDATE`,
						A.`DATEFIX`,
						A.`SHOWVISITS`,
						A.`SHOWDATE`,
						B.`NAME` AS `IMAGE_NAME`,
                        B.`URL` AS `IMAGE_URL`,
                        B.`DESCR` AS `IMAGE_DESCR`,
                    	X.`ALIGN` AS `IMAGE_ALIGN`,
                    	GROUP_CONCAT(DISTINCT C.`option_id_fk` ORDER BY C.`option_id_fk` ASC SEPARATOR ',') AS `OPTIONS`
					FROM
						((`dynpg_cms_article` AS A LEFT JOIN `dynpg_cms_leadimages` AS X
						 ON A.`id` = X.`AID`) LEFT JOIN `dynpg_cms_images` AS B
						 ON B.`id` = X.`IID`) LEFT JOIN `dynpg_cms_article_options` AS C
						 ON A.`id` = C.`article_id_fk`
					WHERE
						A.`ISPUBLIC` = 'TRUE'
					GROUP BY
						A.`id` ASC
					ORDER BY A.`GROUP` ASC, " . implode(', ' , array_map(create_function('$a, $b', 'return "`".mysql_real_escape_string(str_replace(\'`\', \'\', $a))."` ".((strtoupper($b) == \'ASC\')?\'ASC\':\'DESC\')."";'), $this->parameters['order_articles'], $this->parameters['sort_articles']))
			);
            while ( $row = mysql_fetch_assoc($_) ) {
                // add only articles from returned groups
                if (in_array($row['GROUP'], $groupKeys)){
                    if (!empty($this_articles[(integer)$row['GROUP']])){
                        $this_articles[(integer)$row['GROUP']][] = $row;
                    } else {
                        $this_articles[(integer)$row['GROUP']][0] = $row;
                    }
                }
            }
            // $mem_new = round((memory_get_usage() - $mem_old) / 1024, 2);
            // echo "<p style=\"color: red\">" . round(memory_get_usage() / 1024, 2) . " ({$mem_new}) Kbytes</p>";

		// calculate minimal depth
			$minDepth = max($minDepth, $this->parameters['min_depth']);

		// get information about current selected group
	        $root_parents = $menuProc->allParents($self);
	        $next_parents = $root_parents;
	        array_push($next_parents, $self);

        // init vars
			$menuStr 		 = '';
			$listOfGroups    = '';
	        $leaveDepth      = 0;
	        $skip_counter	 = 0;
	        $lastDepth_added = 0;
	        $listIndexGroups = 0;
	        $listIndexItems  = 0;
            $hadArticles = 'False';
            $hadSuccessors = 'False';
            $hadArticlesDepth1 = 'False';
            $hadSuccessorsDepth1 = 'False';
            $hadArticlesDepth2 = 'False';
            $hadSuccessorsDepth2 = 'False';
            $hadArticlesDepth3 = 'False';
            $hadSuccessorsDepth3 = 'False';
	        $template_row[0] = $this->template->getSubTemplate('listOfGroups', dirname( __FILE__) . '/templates/' . $this->dynpg_instance->_PlugIn['dpgdynmenu']['template'] . '/menu.tpl');

        foreach ( $menu as $key => $sub ) {
            // if group depth is too low, skip
            if ($this->parameters['min_depth'] && $sub[1] < $this->parameters['min_depth']) {
            	$skip_counter++;
				continue;
            }


            // check if current group is locked by the community, so hide
            if ($this->locked($sub[0]['id'])) {
            	$skip_counter++;
				continue;
            }


            // all parents of the current group
			$this_parents = $menuProc->allParents($sub[0]['id']);


            // check if current element is last one of one level so we could realize
            // a folded/unfolded management with CSS
            $leaveDepth = $key > 0? ( $menu[ $key - 1 - $skip_counter ][1] - $sub[1] ): 'null';


            // reset $skip_counter
            $skip_counter = 0;


            // level selection
            $sheetState  = ( // each root elements
                             !count($this_parents)

                             // each element in levels before
                             || ((count($menuProc->arraySameElements($root_parents, $this_parents))==$sub[1]) && $currentDepth > $sub[1])

                             // each elements in same level and same group
                             || ( ($root_parents == $this_parents) && $currentDepth == $sub[1] )

                             // each elements in next level
                             || $this_parents == $next_parents )

                             ? 'expand' : 'collapse';


            // check if current item can be found inside the explicit implode array
			if (count($menuProc->arraySameElements($this->parameters['implode_subtree'], $this_parents)) > 0) {
				$sheetState = 'collapse';
            }


            // check if current item can be found inside the explicit expand array
			if ( in_array($sub[0]['id'], $this->parameters['expand_subtree']) || $menuProc->isDirectParentFrom($this->parameters['expand_subtree'], $sub[0]['id']) ) {
				$sheetState = 'expand';
            }


            // only load collapsed items if full_load param is boolean true
			if ( $this->parameters['full_load'] === false ) {
                if ( $sheetState == 'collapse' ) {
                    $menu[ $key ][1] = $key > 0? ( $menu[ $key - 1 ][1] ): 0;
                    continue;
                }
            }


            // elements in the same level will be marked as 'collapse' instead of 'expand'
			if ( // is real expand mode enabled?
				 $this->parameters['real_expand']

				 // is current sheet state expand (otherwise we have to do nothing)?
				 && $sheetState == 'expand'

				 // is current item NOT inside the branch to the current item?
				 && !in_array($sub[0]['id'], $root_parents)

				 // each root elements
				 && (!count($this_parents)

				 // each elements in same level and same group
				 || !(($root_parents == $this_parents) && $currentDepth == $sub[1]))) {

				 $sheetState = 'collapse';
            }


            // save last added depth to handle ignored groups
			$lastDepth_added = $sub[1];


            // generate some template variables
			$listitem_id = strtoupper( md5( uniqid('', true) ) );
            $linkQuery   = '';
                           /*dynpg_insert_urlheader("read_article=(\w+)",
                                dynpg_insert_urlheader("read_group=(\w+)",
                                    dynpg_insert_urlheader("read_category=(\w+)",
										dynpg_insert_urlheader(
												"t=(\w+)"
										)
									)
                                )
                           );*/


            // link query generation
			if ( !empty($sub[0]['REFID']) ) {
				$t_query = "t=" . urlencode($sub[0]['REF_ID_TITLE']);
			} else {
				$t_query = "";
			}

			if ( $sub[2] ) {
                $real = is_null($this->parameters['href_page_groups'])? basename($_SERVER['PHP_SELF']) : ((preg_match('/^http/', $this->parameters['href_page_groups']) == 1)?$this->parameters['href_page_groups']:$this->dynpg_instance->PathToRoot . $this->parameters['href_page_groups']);
                $link = $real . '?' . $this->dynpg_instance->dynPG_getContent_organizeUrlQueryString("$linkQuery&$t_query&read_category=-emptyValue-");
            } else {
                $real = is_null($this->parameters['href_page_articles'])? basename($_SERVER['PHP_SELF']) : ((preg_match('/^http/', $this->parameters['href_page_articles']) == 1)?$this->parameters['href_page_articles']:$this->dynpg_instance->PathToRoot . $this->parameters['href_page_articles']);
                $link = $real . '?' . $this->dynpg_instance->dynPG_getContent_organizeUrlQueryString("$linkQuery&$t_query&read_group=-emptyValue-");
            }

			$href_subpages_def = false;

            if ( array_key_exists($sub[0]['id'], $this->parameters['href_subpages']) ) {
            	$link = (is_array($this->parameters['href_subpages'][$sub[0]['id']])?

						( !empty($this->parameters['href_subpages'][$sub[0]['id']]['page'])?

							((preg_match('/^http/', $this->parameters['href_subpages'][$sub[0]['id']]['page']) == 1)?$this->parameters['href_subpages'][$sub[0]['id']]['page']:$this->dynpg_instance->PathToRoot.$this->parameters['href_subpages'][$sub[0]['id']]['page']):

							call_user_func(create_function('', 'trigger_error("Unconform usage of parameter &gt;href_subpages&lt;: Array-parameter '.
								'<strong>must</strong> have keys \"page\" and \"successors\"!", E_USER_NOTICE); return basename($_SERVER["PHP_SELF"]);'))):

						 ((preg_match('/^http/', $this->parameters['href_subpages'][$sub[0]['id']]) == 1)?$this->parameters['href_subpages'][$sub[0]['id']]:$this->dynpg_instance->PathToRoot.$this->parameters['href_subpages'][$sub[0]['id']]));

				if ( strpos($link, '{query}') ) {
            		$link = str_replace('{query}', '?' . $this->dynpg_instance->dynPG_getContent_organizeUrlQueryString("$linkQuery&$t_query&" . ( $sub[2]? "read_category": "read_group" ) . "=-emptyValue-"), $link);
            	}

				$href_subpages_def = true;
            } else {
                if ( count( $_ = $menuProc->arraySameElements(array_keys($this->parameters['href_subpages']), $this_parents) ) ) {
                    $selID = call_user_func( create_function('$mP, $aR, $curObj', '$g = Array(-1, -1); foreach($aR as $k_ => $v_) { $g = (($__ = count($mP->allParents($v_))) > $g[0] && in_array($v_, array_keys($curObj->parameters["href_subpages"])))? Array($__, $v_): $g; } return $g[1];'), $menuProc, $this_parents, $this );

					unset($lambda_page);
                    unset($lambda_successors);

					if ( is_array($this->parameters['href_subpages'][$selID]) ) {
						extract($this->parameters['href_subpages'][$selID], EXTR_PREFIX_ALL, 'lambda');

                    	if ( empty($lambda_page) || !isset($lambda_successors) ) {
                    		$lambda_page = basename($_SERVER['PHP_SELF']);
                    		$lambda_successors = false;
                    		trigger_error('Unconform usage of parameter &gt;href_subpages&lt;: Array-parameter <strong>must</strong> have keys "page" and "successors"!', E_USER_NOTICE);
                    	}
                    } else {
                    	$lambda_page = $this->parameters['href_subpages'][$selID];
                    	$lambda_successors = true;
                    }

					if ($lambda_successors) {
						// $link = $this->dynpg_instance->PathToRoot . $lambda_page . '?' . $this->dynpg_instance->dynPG_getContent_organizeUrlQueryString("$linkQuery&$t_query&" . ( $sub[2]? "read_category": "read_group" ) . "=-emptyValue-");
						$link = ((preg_match('/^http/', $lambda_page) == 1)?$lambda_page:$this->dynpg_instance->PathToRoot . $lambda_page);
					}

					if ( strpos($link, '{query}') ) {
	            		$link = str_replace('{query}', '?' . $this->dynpg_instance->dynPG_getContent_organizeUrlQueryString("$linkQuery&$t_query&" . ( $sub[2]? "read_category": "read_group" ) . "=-emptyValue-"), $link);
	            	}

					/*if ( strpos($link, '{query}') ) {
	            		$link = str_replace('{query}', '', $link);
	            	}*/

					$href_subpages_def = true;
                }
            }
            $real = is_null($this->parameters['href_page_detail'])? basename($_SERVER['PHP_SELF']) : ((preg_match('/^http/', $this->parameters['href_page_detail']) == 1)?$this->parameters['href_page_detail']:$this->dynpg_instance->PathToRoot . $this->parameters['href_page_detail']);


            // parse articles if necessary
            $listOfArticles  = '';
            $listIndexArticles = 0;
            $template_row[1] = $this->template->getSubTemplate('listOfArticles', $template_row[0], false);

            if ($this->parameters['load_articles'] && !in_array($sub[0]['id'], $this->parameters['implode_subtree'])) {
				// check if articles have to be added
                $skip_articles = false;
				if ($this->parameters['full_load_articles'] === false && $this->parameters['full_load'] === false && !in_array($sub[0]['id'], $root_parents) && !in_array($sub[0]['id'], $this->parameters['expand_subtree'])) {
                    if ($self != $sub[0]['id']) {
						$skip_articles = true;
                    }
	            }
	            if ($this->parameters['full_load_articles'] && $currentDepth > $sub[1]) {
	            	$skip_articles = true;
	            }

	            // check if articles exist and add them
				if (!$skip_articles && !empty($this_articles[(integer)$sub[0]['id']])) {
					foreach ($this_articles[(integer)$sub[0]['id']] as $article) {
		            	// check if current group is locked by the community, so hide
                            if ($this->locked($article['id'], true))
                                continue;

                        // check if article has to be skipped by options
		            		$options = explode(',', $article['OPTIONS']);
		            		if (in_array(1, $options))
		            			continue;

						// increment article counter
                            $listIndexArticles++;

						// check if limit is reached
                            if ($this->parameters['limit_articles'] !== false && $this->parameters['limit_articles'] < $listIndexArticles)
    							break;

    					// prepare template variables
                            $ARTICLE_TITLE_FOR_URL = $article['TITLE'];
                            $article['IMAGE_URL'] = $_general_image_url . $article['IMAGE_URL'];
                            $article = dynPG_Dynamic_Menu::array_change_key_case($article, CASE_UPPER);
                			foreach ($article as $key => $value) {
                                if ($key != 'LEAD') {
                                    $article["{$key}_NOTAGS"] = strip_tags($value);
                                    $article["{$key}_NOTAGS_NOHTML"] = dynpg_htmlentities_rigid($article["{$key}_NOTAGS"]);
                                    $article["{$key}_NOHTML"] = dynpg_htmlentities_rigid($value);
                                    $article["{$key}"] = dynpg_htmlentities($value);
                                } else {
                                    if (!empty($value) && $this->parameters['parse_article_lead'] !== false) {
                                        $article['LEAD_EM'] = dynpg_htmlentities_rigid($this->dynpg_instance->fix_image_path($article['LEAD'], true),ENT_COMPAT,true);

                                        $article['LEAD'] = PerformTextFormat($article['LEAD']);
                                        $article['LEAD'] = ReplaceDummyCode($article['LEAD'], FALSE, 0);

                                        $article['LEAD_NOHTML'] = dynpg_htmlentities_rigid(strip_tags($article['LEAD']));
                                        $article['LEAD'] = dynpg_htmlentities($article['LEAD']);
                                    } else {
                                        $article['LEAD_EM'] = dynpg_htmlentities_rigid($this->dynpg_instance->fix_image_path($article['LEAD'], true),ENT_COMPAT,true);
                                        $article['LEAD_NOHTML'] = dynpg_htmlentities_rigid(strip_tags($article['LEAD']));
                                    }
                                }
                            }

						// prepare edit mode
                            // calculate current editModeID
                            $editModeID = strtoupper(uniqid('EID')) . '_ID' . $article['ID'];

                            // rewrite fields with edit mode code if edit mode is enabled
                            $article['LEAD_NOEDIT'] = $article['LEAD'];
                            $article['TITLE_NOEDIT'] = $article['TITLE'];
                            $article['LEAD'] = $this->dynpg_instance->includeEditMode('ALED', $editModeID, $article['LEAD'], 'long-text', $article['LEAD_EM']);
                            $article['TITLE'] = $this->dynpg_instance->includeEditMode('ATIT', $editModeID, $article['TITLE'], 'small-text', $article['TITLE_NOHTML'], true);

						// add to listing
                            $listIndexItems++;
                            $listOfArticles .= $this->template->fillTemplate(
                                $template_row[1],
                                false,
                                array_merge(
                                    $article,
                                    Array (
                                      // depth of currently selected element
                		  			  'M_CURRENT_DEPTH' => $currentDepth,

                                      'M_HAS_SUCCESSORS' => 'False',

                                      'M_DEPTH' => $sub[1] + 1,

                                      'M_ACTIVE' => $subE && $article['id'] == $subE  ? 'True' : 'False',

                                      'M_MODE' => $subE? 'article': 'group',

                          			  'M_SELF' => $self,

                                      'M_STATE' => ($self == $sub[0]['id'] || in_array($sub[0]['id'], $this->parameters['expand_subtree'])) ? 'expand' : 'collapse',

                                      'M_UNIQUE_ID' => strtoupper( md5( uniqid('', true) ) ),

                                      'M_LINK' => dynPG_Dynamic_Menu::htmlrealpath(($href_subpages_def? preg_replace('/\?.*$/', '', $link): $real) . '?' . $this->dynpg_instance->dynPG_getContent_organizeUrlQueryString("$linkQuery&t=".urlencode($ARTICLE_TITLE_FOR_URL)."&read_article=" . $article['ID'])),

                                      'M_LINK_RELATIVE' => ($href_subpages_def? preg_replace('/\?.*$/', '', $link): $real) . '?' . $this->dynpg_instance->dynPG_getContent_organizeUrlQueryString("$linkQuery&t=".urlencode($ARTICLE_TITLE_FOR_URL)."&read_article=" . $article['ID']),

                                      'M_ISGROUP' => 'False',

                                      'M_LIST_INDEX' => $listIndexItems

                                    )
                                )
                            );
		            } // end for
		        } // end if show articles
	        } // end if imploded


            // replace article list in template
            $template_row[2] = $this->template->replaceSubTemplate('listOfArticles', $listOfArticles, $template_row[0], false);


            // prepare data to be inserted
                // prepare template variables
                    $link_relative = $link;
                    $link = dynPG_Dynamic_Menu::htmlrealpath($link);
                	$sub[0]['IMAGE_URL'] = $_general_image_url . $sub[0]['IMAGE_URL'];
        			$thisRow = dynPG_Dynamic_Menu::array_change_key_case($sub[0], CASE_UPPER);
        			foreach ($thisRow as $key => $value) {
                        if ($key != 'SHORTTEXT') {
                            $thisRow["{$key}_NOTAGS"] = strip_tags($value);
                            $thisRow["{$key}_NOTAGS_NOHTML"] = dynpg_htmlentities_rigid($thisRow["{$key}_NOTAGS"]);
                            $thisRow["{$key}_NOHTML"] = dynpg_htmlentities_rigid($value);
                            $thisRow["{$key}"] = dynpg_htmlentities($value);
                        } else {
                            if (!empty($value) && $this->parameters['parse_group_shorttext'] !== false) {
                                $thisRow['SHORTTEXT_EM'] = dynpg_htmlentities_rigid($this->dynpg_instance->fix_image_path($thisRow['SHORTTEXT'], true),ENT_COMPAT,true);

                                $thisRow['SHORTTEXT'] = PerformTextFormat($thisRow['SHORTTEXT']);
                                $thisRow['SHORTTEXT'] = ReplaceDummyCode($thisRow['SHORTTEXT'], FALSE, 0);

                                $thisRow['SHORTTEXT_NOHTML'] = dynpg_htmlentities_rigid(strip_tags($thisRow['SHORTTEXT']));
                                $thisRow['SHORTTEXT'] = dynpg_htmlentities($thisRow['SHORTTEXT']);
                            } else {
                                $thisRow['SHORTTEXT_EM'] = dynpg_htmlentities_rigid($this->dynpg_instance->fix_image_path($thisRow['SHORTTEXT'], true),ENT_COMPAT,true);
                                $thisRow['SHORTTEXT_NOHTML'] = dynpg_htmlentities_rigid(strip_tags($thisRow['SHORTTEXT']));
                            }
                        }
                    }

                // prepare edit mode
                    // calculate current editModeID
                    $editModeID = strtoupper(uniqid('EID')) . '_ID' . $sub[0]['id'];

                    // rewrite fields with edit mode code if edit mode is enabled
                    $thisRow['SHORTTEXT_NOEDIT'] = $thisRow['SHORTTEXT'];
                    $thisRow['NAME_NOEDIT'] = $thisRow['NAME'];
                    $thisRow['SHORTTEXT'] = $this->dynpg_instance->includeEditMode('GSHT', $editModeID, $thisRow['SHORTTEXT'], 'long-text', $thisRow['SHORTTEXT_EM']);
                    $thisRow['NAME'] = $this->dynpg_instance->includeEditMode('GNAM', $editModeID, $thisRow['NAME'], 'menu-text', $thisRow['NAME_NOHTML'], true);


            // insert data
    			$hasArticles = ((empty($listOfArticles) || $thisRow['PUBLIC'] == 4) ? 'False': 'True');
    			$listIndexGroups++;
                $listOfGroups .= $this->template->fillTemplate(
                    $template_row[2],
                    false,
                    array_merge(
                        $thisRow,
                        Array (
                          // depth of currently selected element
                		  'M_CURRENT_DEPTH' => $currentDepth,

                          'M_HAS_SUCCESSORS' => $sub[2] ? 'True' : 'False',

                          'M_DEPTH' => $sub[1],

                          'M_LEAVE_DEPTH' => $leaveDepth,

                          'M_ACTIVE' => ($self == $sub[0]['id'] || in_array($sub[0]['id'], $root_parents)) ? 'True' : 'False',

                          'M_MODE' => $subE? 'article': 'group',

                          'M_SELF' => $self,

                          'M_STATE' => $sheetState,

                          'M_UNIQUE_ID' => $listitem_id,

                          'M_LINK' => str_replace('-emptyValue-', $thisRow['ID'], $link),

                          'M_LINK_ISEXTERNAL' => ((strpos($link, $this->dynpg_instance->PathToRoot) !== false)?'False':'True'),

                          'M_LINK_RELATIVE' => str_replace('-emptyValue-', $thisRow['ID'], $link_relative),

                          'M_ISGROUP' => 'True',

                          'M_LIST_INDEX' => $listIndexGroups,

                          'M_HAS_ARTICLES' => $hasArticles,

                          'M_ROOT_HAD_ARTICLES' => $hadArticles,

                          'M_ROOT_HAD_SUCCESSORS' => $hadSuccessors,

                          'M_DEPTH1_HAD_ARTICLES' => $hadArticlesDepth1,

                          'M_DEPTH1_HAD_SUCCESSORS' => $hadSuccessorsDepth1,

                          'M_DEPTH2_HAD_ARTICLES' => $hadArticlesDepth2,

                          'M_DEPTH2_HAD_SUCCESSORS' => $hadSuccessorsDepth2,

                          'M_DEPTH3_HAD_ARTICLES' => $hadArticlesDepth3,

                          'M_DEPTH3_HAD_SUCCESSORS' => $hadSuccessorsDepth3
                        )
                    )
                );


            // save states
                if ($sub[1] === 0) {
                    $hadArticlesDepth3 = false;
                    $hadArticlesDepth2 = false;
                    $hadArticlesDepth1 = false;
                    $hadSuccessorsDepth1 = false;
                    $hadSuccessorsDepth2 = false;
                    $hadSuccessorsDepth3 = false;
                    $hadArticles = $hasArticles;
                    $hadSuccessors = ((($sub[2]) && ($thisRow['PUBLIC'] == 1 || $thisRow['PUBLIC'] == 4)) ? 'True' : 'False');
                }
                if ($sub[1] === 1) {
                    $hadArticlesDepth3 = false;
                    $hadArticlesDepth2 = false;
                    $hadSuccessorsDepth2 = false;
                    $hadSuccessorsDepth3 = false;
                    $hadArticlesDepth1 = $hasArticles;
                    $hadSuccessorsDepth1 = ((($sub[2]) && ($thisRow['PUBLIC'] == 1 || $thisRow['PUBLIC'] == 4)) ? 'True' : 'False');
                }
                if ($sub[1] === 2) {
                    $hadArticlesDepth3 = false;
                    $hadSuccessorsDepth3 = false;
                    $hadArticlesDepth2 = $hasArticles;
                    $hadSuccessorsDepth2 = ((($sub[2]) && ($thisRow['PUBLIC'] == 1 || $thisRow['PUBLIC'] == 4)) ? 'True' : 'False');
                }
                if ($sub[1] === 3) {
                    $hadArticlesDepth3 = $hasArticles;
                    $hadSuccessorsDepth3 = ((($sub[2]) && ($thisRow['PUBLIC'] == 1 || $thisRow['PUBLIC'] == 4)) ? 'True' : 'False');
                }
        }
        unset($this_articles);

        // parse the menu template
            $menuHTML = $this->template->fillTemplate(
            	$this->template->fillTemplate(
            		$this->template->replaceSubTemplate('listOfGroups', $listOfGroups, dirname( __FILE__) . '/templates/' . $this->dynpg_instance->_PlugIn['dpgdynmenu']['template'] . '/menu.tpl'),
            		false,
                    Array(
            			// last shown element, add leave depth html elements
            			'M_LEAVE_DEPTH' => ($lastDepth_added - 1 - $minDepth),

                        'M_ROOT_HAD_ARTICLES' => $hadArticles,

                        'M_ROOT_HAD_SUCCESSORS' => $hadSuccessors,

                        'M_DEPTH1_HAD_ARTICLES' => $hadArticlesDepth1,

                        'M_DEPTH1_HAD_SUCCESSORS' => $hadSuccessorsDepth1,

                        'M_DEPTH2_HAD_ARTICLES' => $hadArticlesDepth2,

                        'M_DEPTH2_HAD_SUCCESSORS' => $hadSuccessorsDepth2,

                        'M_DEPTH3_HAD_ARTICLES' => $hadArticlesDepth3,

                        'M_DEPTH3_HAD_SUCCESSORS' => $hadSuccessorsDepth3
                    )
                ),
                false
            );

		// insert dynamic editMode layers if necessary
            $this->dynpg_instance->_editModeLayerAppendLock = false;
            $this->dynpg_instance->replace_editMode_layer($menuHTML);

		return $menuHTML;
    }

    function locked($id, $article = false) {
        // check if this feature is enabled
            if (!$this->parameters['hide_locked_groups_by_community'])
                return false;

        // check if community plugin is installed
            if (!$this->dynpg_instance->plugInExists('DPGcommunity'))
                return false;

    	// fetch community object
            $dpgcommunity = $this->dynpg_instance->call_PlugIn('DPGcommunity');

    	// check if locking feature is enabled
            if (!$dpgcommunity->parameters['enable_locking_feature'])
                return false;

		// check lock for current $id
            if ($article) {
                $lock = $dpgcommunity->dpglock_verify(
    				false,
    				Array($id),
    				true
    			);
            	return ($lock !== C_COMMUNITY_NOLOCK && $lock !== C_COMMUNITY_FREE);
    		} else {
    			$lock = $dpgcommunity->dpglock_verify(
    				true,
    				array_merge(Array($id), $this->dynpg_instance->get_DynPG_allParents_Of($id)),
    				true
    			);
            	return ($lock !== C_COMMUNITY_NOLOCK && $lock !== C_COMMUNITY_FREE);
    		}
    }

    function destroy_dynPG_Dynamic_Menu() {
        // destructor //
    }

}

?>